/*****************************************************************************
 * vlvc_frame.cpp: VLC's graphical module for VLVC
 *****************************************************************************
 * Copyright (C) 2005 - 2007 the VLVC team
 * $Id: vlvc_frame.cpp 
 *
 * Authors: Julien Arnaud <arnaud_j@epitech.net>
 *          Julien Cupillard <cupill_j@epitech.net>
 *          Arnaud Deredec <derede_a@epitech.net>
 *          Florian Gustin <gustin_f@epitech.net>
 *          Guillaume Martin <martin_u@epitech.net>
 *          Florent Nguyen <nguyen_c@epitech.net>
 *
 * This software has been written in EPITECH <http://www.epitech.net>
 * EPITECH is computer science school in Paris - FRANCE -
 * under the direction of Flavien Astraud <http://www.epita.fr/~flav>,
 * Jerome Landrieu and Launiau Severin.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
 *****************************************************************************/

/*****************************************************************************
 * Preamble
 *****************************************************************************/

#include <wx/statline.h>
#include "wxwidgets.hpp"
#include <wx/tooltip.h>
#include <vlc/vout.h>
#include "video.hpp"
#include "vlvc_frame.hpp"
#include "vlc_vlvc_error.h"

BEGIN_EVENT_TABLE( VlvcFrame, wxFrame )
    EVT_BUTTON( SendEvent, VlvcFrame::OnButtonSendPress )
    EVT_BUTTON( PlayEvent, VlvcFrame::OnButtonPlayPress )
    EVT_BUTTON( StopEvent, VlvcFrame::OnButtonStopPress )
    EVT_LISTBOX_DCLICK( UserList, VlvcFrame::OnUserListDblClick )
    EVT_CLOSE( VlvcFrame::OnCloseWindow )
    EVT_TEXT_ENTER( PressEnterEvent, VlvcFrame::OnButtonSendPress )
    EVT_COMMAND_ENTER( VLVC_SHOW_MESSAGE_EVENT, VlvcFrame::OnMessage )
    EVT_COMMAND_ENTER( VLVC_ADD_USER_EVENT, VlvcFrame::OnAddUser )
    EVT_COMMAND_ENTER( VLVC_DEL_USER_EVENT, VlvcFrame::OnDelUser )
    EVT_COMMAND_ENTER( VLVC_CHAT_MESSAGE_EVENT, VlvcFrame::OnChatMessage )
END_EVENT_TABLE()

/*****************************************************************************
 * Name: OnIncomingChatMessage
 *
 * Parameters: 
 *             int i_id_user        : Id of the user who sent a message
 *             char *psz_message    : Message
 *             int i_message_type   : type of the message
 *             void* inst           : The instance of VlvcFrame
 * Returns:
 *             void
 *
 * Description: 
 *              This function is called by the core module to call the class 
 *              method. Used to display a message in the chat area.
 *****************************************************************************/
static void OnIncomingChatMessage( int id, const char *psz_message, 
				   int i_message_type, void *inst )
{
    static VlvcFrame *p_this = NULL;

    if( inst )
        p_this = ( VlvcFrame * )inst;
    if( p_this && !inst )
        p_this->OnIncomingChatMessage( id, psz_message, i_message_type );
}

/*****************************************************************************
 * Name: OnNbClientChange
 *
 * Parameters: 
 *             int nb : Id of the user who just signed in or left
 *             void* inst : The instance of VlvcFrame
 *
 * Returns:
 *             void
 *
 * Description: 
 *              This function is called by the core module to call the class 
 *              method. Used to notify a client has joined or left.
 *****************************************************************************/
static void OnNbClientChange( int id, void *inst )
{
    static VlvcFrame *p_this = NULL;

    if( inst )
        p_this = ( VlvcFrame * )inst;
    if( p_this && !inst )
        p_this->OnNbClientChange( id );
}

/*****************************************************************************
 * Name: OnStopVlvc
 *
 * Parameters: 
 *             void* inst : The instance of VlvcFrame
 *
 * Returns:
 *             void
 *
 * Description: 
 *              This function is called by the core module to call the class 
 *              method. Called when the core module is stopped.
 *****************************************************************************/
static void OnStopVlvc( void *inst )
{
    static VlvcFrame *p_this = NULL;

    if( inst )
        p_this = ( VlvcFrame * )inst;
    if( p_this && !inst )
        p_this->OnStopVlvc( );
}

/*****************************************************************************
 * Name: OnCoreModuleMessage
 *
 * Parameters: 
 *             char *psz_message: message to display
 *             int i_critical: critical level
 *             void *inst: the instance of VlvcFrame
 *
 * Returns:
 *             void
 *
 * Description: 
 *             This function is called by the core module to call the class 
 *             method. Used to display a message box.
 *****************************************************************************/
static void OnCoreModuleMessage( char *psz_message, int i_critical, void *inst )
{
    static VlvcFrame *p_this = NULL;

    if( inst )
        p_this = ( VlvcFrame * )inst;
    if( p_this && !inst )
        p_this->OnCoreModuleMessage( psz_message, i_critical );
}

/*****************************************************************************
 * Name: VlvcFrame::VlvcFrame
 *
 * Parameters: 
 *             intf_thread_t *p_intf : Thread interface instance
 *             wxWindow *p_parent : Parent window containing this frame
 *
 * Description: 
 *              VlvcFrame Constructor
 *****************************************************************************/
VlvcFrame::VlvcFrame( intf_thread_t *p_intf, wxWindow *p_parent ) :
    wxFrame( p_parent, -1, wxT(VLVC_WINDOW_NAME), wxDefaultPosition,
    wxDefaultSize, wxCAPTION | wxMINIMIZE_BOX | wxRESIZE_BORDER )
{
    memset( (void*)p_tab_status, VLVC_USER_STATUS_OFF, VLVC_MAX_USERS );
    p_user_list = NULL;
    ::OnNbClientChange( 0, (void*) this );
    ::OnStopVlvc( (void*) this );
    ::OnCoreModuleMessage( NULL, 0, (void *) this );
    ::OnIncomingChatMessage( 0, NULL, 0, (void *) this );

    vlc_mutex_init( p_intf->p_vlc, &this->messageLock );
    //vlc_mutex_init( p_intf->p_vlc, &this->chatMessageLock );
    //this->b_chatMessageLocked = VLC_FALSE;
    this->b_messageLocked = VLC_FALSE;
    
    this->p_intf = p_intf;
    this->p_parent = p_parent;
    p_vlvc = vlvc_New( p_intf );
    b_first_added = VLC_FALSE;
    InitControls();

    while ( p_vlvc->b_initialized == VLC_FALSE ) msleep( 10000 );
    vlvc_SetCallbackChangeNbUsers( p_vlvc, ::OnNbClientChange );
    vlvc_SetCallbackStop( p_vlvc, ::OnStopVlvc );
    vlvc_SetCallbackCoreModuleMessage( p_vlvc, ::OnCoreModuleMessage );
    vlvc_SetCallbackChatMessage( p_vlvc, ::OnIncomingChatMessage );
    if( vlvc_Start( p_vlvc ) != VLVC_SUCCESS )
    {
        msg_Err( p_vlvc, "vlvc : Core module not started\n" );
        ShowMessage( p_vlvc_error_messages[p_vlvc->i_last_error], VLVC_CRITICAL_LEVEL_MAX );
    }
}

/*****************************************************************************
 * Name: VlvcFrame::~VlvcFrame
 *
 * Description: 
 *              VlvcFrame Destructor
 *****************************************************************************/
VlvcFrame::~VlvcFrame()
{
    vlvc_SetCallbackChangeNbUsers( p_vlvc, NULL );
    vlvc_SetCallbackStop( p_vlvc, NULL );
    vlvc_SetCallbackCoreModuleMessage( p_vlvc, NULL );
    vlvc_Delete( p_vlvc );
}

/*****************************************************************************
 * Name: VlvcFrame::OnMessage
 *
 * Parameters: 
 *              wxCommandEvent &evt: event object
 *
 * Returns:
 *              void
 *
 * Description: 
 *              Event generated to display a message box
 *****************************************************************************/
void VlvcFrame::OnMessage(wxCommandEvent &evt) 
{
    MessageEventData *p_data = (MessageEventData *)evt.GetClientData();
    if ( p_data )
    {
        char *psz_title = VLVC_CRITICAL_TITLE_LOW;
        long style = wxOK;
        if ( p_data->i_critical > VLVC_CRITICAL_LEVEL_QUIT_LIMT )
        {
            psz_title = VLVC_CRITICAL_TITLE_HIGH;
            style |= wxICON_ERROR;
        }
        else
        {
            style |= wxICON_INFORMATION;
        }
        wxMessageDialog messageDialog( this, wxString( wxU( _( p_data->psz_message ) ) ),
                                       wxString( wxU( _( psz_title ) ) ), 
                                       style, wxPoint(100, 100) );
        messageDialog.ShowModal();
        if ( p_data->i_critical > VLVC_CRITICAL_LEVEL_QUIT_LIMT )
        {
            msg_Err( p_vlvc, "vlvc : Core module error\n" );
            vlvc_Stop( p_vlvc );
	}
    }
    this->b_messageLocked = VLC_FALSE;
}

/*****************************************************************************
 * Name: VlvcFrame::OnAddUser
 *
 * Parameters: 
 *              wxCommandEvent &evt: event object
 *
 * Returns:
 *              void
 *
 * Description: 
 *              Event generated to add a user in the list.
 *****************************************************************************/
void VlvcFrame::OnAddUser(wxCommandEvent &evt)
{
    int         i_inserted;
    s_user_info *p_user;
    char        psz_nickname[VLVC_MAX_USER_NAME_LENGTH];
    char        psz_desc[VLVC_MAX_USER_DESC_LENGTH];
    int         i_client_id = (int)evt.GetClientData();
    
    p_user = &tab_user_info[i_client_id];
    memset( psz_nickname, STRING_TERMINATOR, VLVC_MAX_USER_NAME_LENGTH );
    memset( psz_desc, STRING_TERMINATOR, VLVC_MAX_USER_DESC_LENGTH );
    vlvc_GetUserName( p_vlvc, i_client_id, psz_nickname );
    vlvc_GetUserDesc( p_vlvc, i_client_id, psz_desc );

    /* Set user name */
    if( psz_nickname[0] == STRING_TERMINATOR )
        p_user->name = wxString( wxU( _( VLVC_NO_USER_NAME ) ) );
    else
        p_user->name = wxString( wxU( _( psz_nickname ) ) );
		
    /* Set user description */
    if( psz_desc[0] == STRING_TERMINATOR )
        p_user->desc = wxString( wxU( _( VLVC_NO_USER_DESC ) ) );
    else
        p_user->desc = wxString( wxU( _( psz_desc ) ) );

    /* Set user id */
    p_user->i_id = i_client_id;
    p_tab_status[i_client_id] = VLVC_USER_STATUS_ON;
    
    /* Insert the user in the combo box */
    if( p_user_list != NULL ) 
    {
        i_inserted = p_user_list->Append( p_user->name );
        p_user_list->SetClientData( i_inserted, (void *)p_user );
    }
    /* Notify in the chat window that a new user has joined */
    wxString str = p_user->name + wxU( " " ) + wxU( VLVC_CHAT_MESSAGE_JOIN );
    ChatMessage( str, VLVC_CHAT_SYSTEM_MSG_COLOR );
    this->b_messageLocked = VLC_FALSE;
}

/*****************************************************************************
 * Name: VlvcFrame::OnDelUser
 *
 * Parameters: 
 *              wxCommandEvent &evt: event object
 *
 * Returns:
 *              void
 *
 * Description: 
 *              Event generated to delete a user from the list.
 *****************************************************************************/
void VlvcFrame::OnDelUser(wxCommandEvent &evt)
{
    int         i_nb_elems;
    s_user_info *p_user;
    int         i_client_id = (int)evt.GetClientData();

    for( i_nb_elems = 0; i_nb_elems < p_user_list->GetCount(); i_nb_elems++ )
    {
        p_user = (s_user_info*) p_user_list->GetClientData( i_nb_elems );
        if( p_user->i_id == i_client_id )
        {
            /* Notify in the chat window that a user has left */
            wxString str = p_user->name + wxU( " " ) + wxU( VLVC_CHAT_MESSAGE_LEAVE );
            ChatMessage( str, VLVC_CHAT_SYSTEM_MSG_COLOR );

            /* Remove the entry from the combo box */
            p_user_list->Delete( i_nb_elems );
            p_tab_status[i_client_id] = VLVC_USER_STATUS_OFF;
            break;
        }
    }
    this->b_messageLocked = VLC_FALSE;
}

/*****************************************************************************
 * Name: VlvcFrame::OnChatMessage
 *
 * Parameters: 
 *              wxCommandEvent &evt: event object
 *
 * Returns:
 *              void
 *
 * Description: 
 *              Event generated to display a message in the chat area
 *****************************************************************************/
void VlvcFrame::OnChatMessage(wxCommandEvent &evt)
{
    wxColour msg_color = VLVC_CHAT_USER_MSG_COLOR;
    bool b_show_timestamp = true;
    
    ChatMessageEventData *p_chatMessageData = 
        (ChatMessageEventData *)evt.GetClientData();
    if ( p_chatMessageData != NULL )
    {
        wxString message;
        wxString str;

        message = wxU( p_chatMessageData->psz_message );
        if ( p_chatMessageData->i_type == VLVC_NET_MESSAGE_TYPE_INFO )
        {
	    msg_color = VLVC_CHAT_INFO_MSG_COLOR;
	    str = message;
	    b_show_timestamp = false;
	}
        else
	{
	    wxString user_name = tab_user_info[p_chatMessageData->i_user_id].name;
	    str = user_name + wxU( " : " ) + message;
	}        
        ChatMessage( str, msg_color, b_show_timestamp );
    }
    this->b_messageLocked = VLC_FALSE;
}

/*****************************************************************************
 * Name: VlvcFrame::InitControls
 *
 * Returns:
 *             void
 *
 * Description: 
 *              Creates all the controls of the window
 *****************************************************************************/
void VlvcFrame::InitControls()
{
    wxBoxSizer *p_rootsizer = new wxBoxSizer( wxHORIZONTAL );
    p_rootsizer->Add( GetMainPanel( this ), 1, wxEXPAND | wxALL, 5 );
    this->SetAutoLayout( TRUE );
    this->SetSizerAndFit( p_rootsizer );
    this->Show();
}

/*****************************************************************************
 * Name: VlvcFrame::GetMainPanel
 *
 * Parameters:
 *              wxFrame *parent : the parent frame
 *
 * Returns:
 *              wxPanel*: object corresponding to the main panel
 *
 * Description: 
 *              Creates the main panel.
 *****************************************************************************/
wxPanel *VlvcFrame::GetMainPanel( wxFrame *parent )
{
    wxBoxSizer *p_mainsizer;
    wxPanel    *p_mainpanel;

    p_mainpanel = new wxPanel( parent, -1 );
    p_mainsizer = new wxBoxSizer( wxVERTICAL );
    p_mainsizer->Add( GetTopPanel( p_mainpanel ), 1, wxEXPAND );
    p_mainsizer->Add( GetSendPanel( p_mainpanel ), 0, wxEXPAND );
    p_mainpanel->SetSizerAndFit( p_mainsizer );
    p_mainpanel->SetMinSize( wxSize( VLVC_WINDOW_WIDTH, VLVC_WINDOW_HEIGHT ) );
    return p_mainpanel;
}

/*****************************************************************************
 * Name: VlvcFrame::GetTopPanel
 *
 * Parameters:
 *              wxFrame *parent : the parent frame
 *
 * Returns:
 *              wxPanel*: object corresponding to the top panel
 *
 * Description: 
 *              Creates the top panel.
 *****************************************************************************/
wxPanel *VlvcFrame::GetTopPanel( wxPanel *parent )
{
    wxBoxSizer *p_top_sizer;
    wxPanel    *p_top_panel;

    p_top_panel = new wxPanel( parent, -1 );
    p_top_sizer = new wxBoxSizer( wxHORIZONTAL );
    p_top_sizer->Add( GetChatPanel( p_top_panel ), 1, wxEXPAND );
    p_top_sizer->Add( GetActionPanel( p_top_panel ), 0, wxEXPAND );
    p_top_panel->SetSizerAndFit( p_top_sizer );
    return p_top_panel;
}

/*****************************************************************************
 * Name: VlvcFrame::GetActionPanel
 *
 * Parameters:
 *              wxFrame *parent : the parent frame
 *
 * Returns:
 *              wxPanel*: object corresponding to the action panel
 *
 * Description: 
 *              Creates the action panel
 *****************************************************************************/
wxPanel *VlvcFrame::GetActionPanel( wxPanel *parent )
{
    wxBoxSizer *p_action_sizer;
    wxPanel    *p_action_panel;

    p_action_panel = new wxPanel( parent, -1 );
    p_action_sizer = new wxBoxSizer( wxVERTICAL );
    /*    p_action_sizer->Add( GetCbxPanel( p_action_panel ), 1, wxEXPAND | wxALL, 2 );*/

    /*    p_action_sizer->Add( GetInfoPanel( p_action_panel ), 1, wxEXPAND | wxALL, 2 );*/
    p_action_sizer->Add( GetBtnPanel( p_action_panel ), 0, wxEXPAND | wxALL, 2 );
    p_action_sizer->Add( GetUserPanel( p_action_panel ), 1, wxEXPAND | wxALL, 2 );
    p_action_panel->SetSizerAndFit( p_action_sizer );
    return p_action_panel;
}

/*****************************************************************************
 * Name: VlvcFrame::GetBtnPanel
 *
 * Parameters:
 *              wxFrame *parent : the parent frame
 *
 * Returns:
 *              wxPanel*: object corresponding to the buttons panel
 *
 * Description: 
 *              Creates the buttons panel
 *****************************************************************************/
wxPanel *VlvcFrame::GetBtnPanel( wxPanel *parent )
{
    wxButton         *p_stop_button;
    wxButton         *p_play_button;
    wxPanel          *p_btnpanel;
    wxStaticBox      *p_btnbox;
    wxStaticBoxSizer *p_btnsizer;

    p_btnpanel = new wxPanel( parent, -1 );
    p_btnpanel->SetAutoLayout( TRUE );
    p_btnbox = new wxStaticBox( p_btnpanel, -1, VLVC_FRAME_TEXT_ACTION );
    p_btnsizer = new wxStaticBoxSizer( p_btnbox, wxHORIZONTAL );

    p_play_button = new wxButton( p_btnpanel, PlayEvent, VLVC_BUTTON_TEXT_PLAY );
    p_btnsizer->Add( p_play_button, 0, wxEXPAND, 0 );

    p_stop_button = new wxButton( p_btnpanel, StopEvent, VLVC_BUTTON_TEXT_STOP );
    p_btnsizer->Add( p_stop_button, 0, wxEXPAND, 0 );
    p_btnpanel->SetSizerAndFit( p_btnsizer );
    return p_btnpanel;
}

/*****************************************************************************
 * Name: VlvcFrame::GetCbxPanel
 *
 * Parameters:
 *              wxFrame *parent : the parent frame
 *
 * Returns:
 *              wxPanel*: object corresponding to the combo panel
 *
 * Description: 
 *              Creates the combo panel
 *****************************************************************************/
wxPanel *VlvcFrame::GetUserPanel( wxPanel *parent )
{
    wxPanel          *p_user_panel;
    wxStaticBox      *p_user_box;
    wxStaticBoxSizer *p_user_sizer;

    p_user_panel = new wxPanel( parent, -1 );
    p_user_box = new wxStaticBox( p_user_panel, -1, VLVC_FRAME_TEXT_USER );
    p_user_sizer = new wxStaticBoxSizer( p_user_box, wxVERTICAL );
    p_user_list = new wxListBox( p_user_panel, UserList );
    p_user_sizer->Add( p_user_list, 1, wxEXPAND, 5 );
    p_user_panel->SetSizerAndFit( p_user_sizer );
    return p_user_panel;
}

/*****************************************************************************
 * Name: VlvcFrame::GetChatPanel
 *
 * Parameters:
 *              wxFrame *parent : the parent frame
 *
 * Returns :
 *              wxPanel*: object corresponding to the Chat panel
 *
 * Description: 
 *              Creates the Chat panel.
 *****************************************************************************/
wxPanel *VlvcFrame::GetChatPanel( wxPanel *parent )
{
    wxPanel          *p_chat_panel;
    wxStaticBox      *p_chat_box;
    wxStaticBoxSizer *p_chat_sizer;

    p_chat_panel = new wxPanel( parent, -1 );
    p_chat_box   = new wxStaticBox( p_chat_panel, -1, VLVC_FRAME_TEXT_CHAT );
    p_chat_sizer = new wxStaticBoxSizer( p_chat_box, wxVERTICAL );


    p_chat_textbox = new wxTextCtrl( p_chat_panel, -1,
                                     VLVC_TEXT_BOX_DEFAULT_TEXT,
                                     wxDefaultPosition,
                                     wxSize( VLVC_CHAT_TEXTBOX_WIDTH, VLVC_CHAT_TEXTBOX_HEIGHT ),
                                     wxTE_MULTILINE | wxTE_READONLY | wxTE_RICH );
    p_chat_sizer->Add( p_chat_textbox, 1, wxEXPAND );
    //    p_chat_sizer->Add( GetSendPanel( p_chat_panel ), 0, wxEXPAND );
    p_chat_panel->SetSizerAndFit( p_chat_sizer );
    return p_chat_panel;
}

/*****************************************************************************
 * Name: VlvcFrame::GetSendPanel
 *
 * Parameters:
 *              wxFrame *parent : the parent frame
 *
 * Returns:
 *              wxPanel*: object corresponding to the Send panel
 *
 * Description: 
 *              Creates the Send panel.
 *****************************************************************************/
wxPanel *VlvcFrame::GetSendPanel( wxPanel *parent )
{
    wxPanel          *p_send_panel;
    wxBoxSizer       *p_send_sizer;
    wxButton         *p_send_button;

    p_send_panel = new wxPanel( parent, -1 );
    p_send_sizer = new wxBoxSizer( wxHORIZONTAL );

    /* The textbox where the text is typed in */
    p_entry_textbox = new wxTextCtrl( p_send_panel, PressEnterEvent,
                                     VLVC_TEXT_BOX_DEFAULT_TEXT,
                                     wxDefaultPosition,
                                     wxDefaultSize,
                                     wxTE_PROCESS_ENTER );

    p_entry_textbox->SetMaxLength( VLVC_MAX_SEND_TEXT_LEN );

    /* The send button */
    p_send_button = new wxButton( p_send_panel, SendEvent, VLVC_BUTTON_TEXT_SEND );

    p_send_sizer->Add( p_entry_textbox, 1, wxEXPAND | wxTOP | wxBOTTOM, 4 );
    p_send_sizer->Add( p_send_button, 0, wxEXPAND | wxALL, 4 );
    p_send_panel->SetSizerAndFit( p_send_sizer );
    return p_send_panel;
}

/*****************************************************************************
 * Name: VlvcFrame::OnUserListDblClick
 *
 * Parameters:
 *              wxCommandEvent& event: unused event object.
 *
 * Returns:
 *              void
 *
 * Description: 
 *              Event generated when the user double-clicks on the user list.
 *****************************************************************************/
void VlvcFrame::OnUserListDblClick( wxCommandEvent& WXUNUSED( event ) )
{
    int         i_idx_sel;

    i_idx_sel = p_user_list->GetSelection();
    if( i_idx_sel == wxNOT_FOUND )
    {
        msg_Dbg( p_vlvc, "vlvc : ::OnUserSelect() : No item selected\n" );
        return;
    }

    s_user_info *p_user = (s_user_info*) p_user_list->GetClientData( i_idx_sel );
    wxString strMessage = VLVC_FRAME_TEXT_INFO_USER( p_user->name ) + p_user->desc;
    ChatMessage( strMessage, VLVC_CHAT_DESC_MSG_COLOR, false );
}

/*****************************************************************************
 * Name: VlvcFrame::AddUser
 *
 * Parameters:
 *              int i_client_id: client Id of the client to add
 *
 * Returns:
 *              void
 * 
 * Description: 
 *              This method is called when a user is added to the conversation.
 *              It generates the event needed to add the user in the list.
 *****************************************************************************/
void VlvcFrame::AddUser( int i_client_id )
{
    vlc_mutex_lock( &this->messageLock );
    while( this->b_messageLocked == VLC_TRUE )
        msleep( 10000 );
    this->b_messageLocked = VLC_TRUE;
    vlc_mutex_unlock( &this->messageLock );

    wxCommandEvent evt( wxEVT_COMMAND_ENTER, VLVC_ADD_USER_EVENT );
    evt.SetClientData( (void *)i_client_id );
    wxPostEvent( this, evt );
}

/*****************************************************************************
 * Name: VlvcFrame::DelUser
 *
 * Parameters:
 *              int i_client_id: client Id of the client to delete
 *
 * Returns:
 *              void
 *
 * Description: 
 *              This method is called when a user leaves the conversation.
 *              It generates the event needed to delete the user from the list.
 *****************************************************************************/
void VlvcFrame::DelUser( int i_client_id )
{
    vlc_mutex_lock( &this->messageLock );
    while( this->b_messageLocked == VLC_TRUE )
        msleep( 10000 );
    this->b_messageLocked = VLC_TRUE;
    vlc_mutex_unlock( &this->messageLock );

    wxCommandEvent evt( wxEVT_COMMAND_ENTER, VLVC_DEL_USER_EVENT );
    evt.SetClientData( (void *)i_client_id );
    wxPostEvent( this, evt );
}

/*****************************************************************************
 * Name: VlvcFrame::OnStopVlvc
 *
 * Returns:
 *              void
 *
 * Description: 
 *              Closes the window
 *****************************************************************************/
void VlvcFrame::OnStopVlvc()
{
    Close( true );
}

/*****************************************************************************
 * Name: VlvcFrame::OnNbClientChange
 *
 * Parameters: 
 *              int i_client_id : Id of the client who just signed in or out
 *
 * Returns:
 *              void
 *
 * Description: 
 *              Called when a new user is connected or a user disconnects.
 *****************************************************************************/
void VlvcFrame::OnNbClientChange( int i_client_id )
{
    if( ( i_client_id < 0 ) || ( i_client_id >= VLVC_MAX_USERS ) )
        return;

    switch( p_tab_status[i_client_id] )
    {
    case VLVC_USER_STATUS_OFF:
        AddUser( i_client_id );
        break;

    case VLVC_USER_STATUS_ON:
        DelUser( i_client_id );
        break;

    default:
        break;
    }
}

/*****************************************************************************
 * Name: VlvcFrame::ShowMessage
 *
 * Parameters: 
 *              char *psz_message: the message to be shown
 *              int i_critical: the severity level
 *
 * Returns:
 *              void
 *
 * Description: 
 *              Generates the event needed to display the specified message in
 *              a message box.
 *****************************************************************************/
void VlvcFrame::ShowMessage( char *psz_message, int i_critical )
{
    vlc_mutex_lock( &this->messageLock );
    while( this->b_messageLocked == VLC_TRUE )
        msleep( 10000 );
    this->b_messageLocked = VLC_TRUE;
    vlc_mutex_unlock( &this->messageLock );

    wxCommandEvent evt( wxEVT_COMMAND_ENTER, VLVC_SHOW_MESSAGE_EVENT );
    messageData.i_critical = i_critical;
    strcpy( messageData.psz_message, psz_message );
    evt.SetClientData( &messageData );
    wxPostEvent( this, evt );
}

/*****************************************************************************
 * Name: VlvcFrame::OnCoreModuleMessage
 *
 * Parameters: 
 *              char *psz_message: the message to be shown
 *              int i_critical: the severity level
 *
 * Returns:
 *              void
 *
 * Description: 
 *              Called by the core module to Display a message in a message box.
 *****************************************************************************/
void VlvcFrame::OnCoreModuleMessage( char *psz_message, int i_critical )
{
    ShowMessage( psz_message, i_critical );
}

/*****************************************************************************
 * Name: VlvcFrame::OnButtonSendPress
 *
 * Parameters: 
 *              wxCommandEvent& event : Unused
 *
 * Returns:
 *              void
 *
 * Description: 
 *              Called when the send button is pressed.
 *****************************************************************************/
void VlvcFrame::OnButtonSendPress( wxCommandEvent& event )
{
    wxString    message = p_entry_textbox->GetValue();
    
    message = message.Trim(true);
    message = message.Trim(false);
    if( message.Len() == 0 )
        return;

    const char *psz_message = wxFromLocale( message );
    
    vlvc_NetSendMessage( p_vlvc, psz_message, VLVC_NET_MESSAGE_ALL );
    p_entry_textbox->Clear();
}

/*****************************************************************************
 * Name: VlvcFrame::OnIncomingChatMessage
 *
 * Parameters: 
 *              int i_user_id: user who sent the message
 *              const char *psz_message: message
 *              int i_message_type: type of message
 *
 * Returns:
 *              void
 *
 * Description: 
 *              Callback wrapped by ::OnIncomingChatMessage(), used by the core
 *              module when a new chat message arrives. Generates the event 
 *              needed to display the message in the chat area.
 *****************************************************************************/
void VlvcFrame::OnIncomingChatMessage( int i_user_id, const char *psz_message, 
                                       int i_message_type )
{
    // TODO :implement message type
    vlc_mutex_lock( &this->messageLock );
    while( this->b_messageLocked == VLC_TRUE )
        msleep( 10000 );
    this->b_messageLocked = VLC_TRUE;
    vlc_mutex_unlock( &this->messageLock );
    wxCommandEvent evt( wxEVT_COMMAND_ENTER, VLVC_CHAT_MESSAGE_EVENT );

    memset( this->chatMessageData.psz_message, STRING_TERMINATOR, 
            sizeof( this->chatMessageData.psz_message ) );
    memcpy( this->chatMessageData.psz_message, psz_message, 
            strlen( psz_message ) * sizeof( *psz_message ) );
    this->chatMessageData.i_user_id = i_user_id;
    this->chatMessageData.i_type = i_message_type;

    evt.SetClientData( &this->chatMessageData );
    wxPostEvent( this, evt );
}

/*****************************************************************************
 * Name: VlvcFrame::OnButtonPlayPress
 *
 * Parameters: 
 *              wxCommandEvent& event : Unused
 *
 * Returns:
 *              void
 *
 * Description: 
 *              Called when the play button is pressed.
 *****************************************************************************/
void VlvcFrame::OnButtonPlayPress( wxCommandEvent& event )
{
    int i_sel;

    i_sel = p_user_list->GetSelection();
    if( i_sel == wxNOT_FOUND )
        return;
    s_user_info *p_user = (s_user_info*) p_user_list->GetClientData( i_sel );
    vlvc_PlayUser( p_vlvc, p_user->i_id );
}

/*****************************************************************************
 * Name: VlvcFrame::OnButtonStopPress
 *
 * Parameters: 
 *              wxCommandEvent& event : Unused
 *
 * Returns:
 *              void
 *
 * Description: 
 *              Called when the stop button is pressed.
 *****************************************************************************/
void VlvcFrame::OnButtonStopPress( wxCommandEvent& event )
{
    int i_sel;

    i_sel = p_user_list->GetSelection();
    if( i_sel == wxNOT_FOUND )
        return;
    s_user_info *p_user = (s_user_info*) p_user_list->GetClientData( i_sel );
    vlvc_StopUser( p_vlvc, p_user->i_id );
}

/*****************************************************************************
 * Name: VlvcFrame::OnCloseWindow
 *
 * Parameters: 
 *              wxCloseEvent& event : Unused
 *
 * Returns:
 *              void
 *
 * Description: 
 *              Destroys the window
 *****************************************************************************/
void VlvcFrame::OnCloseWindow( wxCloseEvent& event )
{
    if( !event.CanVeto() )
        Destroy();
}

/*****************************************************************************
 * Name: VlvcFrame::ChatMessage
 *
 * Parameters: 
 *              const char *message: the message to be displayed in the
 *                                   chat textbox
 *              wxColour &color: the color of the message
 *              bool bShowTimeStamp: used to enable or disable the time for this
 *                                   message. Default is true.
 *
 * Returns:
 *              void

 * Description: 
 *              Displays a message in the Chat window
 *****************************************************************************/
void VlvcFrame::ChatMessage( wxString &message, wxColour &color, bool bShowTimeStamp )
{
    if( bShowTimeStamp )
    {
        /* display the timestamp (black) */
        wxDateTime now;
        
        now.Set( wxDateTime::GetTimeNow() );
        wxString str_time = now.Format( wxU( VLVC_TIMESTAMP_FORMAT ) );

        p_chat_textbox->SetDefaultStyle( wxTextAttr( *wxBLACK ) );
        p_chat_textbox->AppendText( str_time );
    }

    /* display the message (user defined color) */
    p_chat_textbox->SetDefaultStyle( wxTextAttr( color ) );
    p_chat_textbox->AppendText( message );
    p_chat_textbox->AppendText( wxU( END_OF_LINE ) );
}
